/*  File: analyser-BpT.cc
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *  Modified by D. Soper on 18 December 2013
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   std includes   */

/*   project headers  */

/*   local headers  */
#include "analyser-BpT.h"

/*   other includes   */


void UserBpT::initfunc()
{
  /*   Create object to calculate the cross sections   */
  UserVoid::phys(0, "Total cross section for b-quark events");
  UserVoid::phys(1, "Cross section in visible bbar pT distribution");
  UserVoid::phys(2, "Cross section in visible bbar y distribution");
  
  /*   Create histograms to calculate the pT distributions   */
  unsigned int NbinsPT = 60;
  double pTmin = 0.0, pTmax = 30.0;
  bool enable_norm = true;
  
  UserPlain::phys(1, "b-bar pT distribution for b-quark events",
                  spacing<hist1d>::linear(NbinsPT, pTmin, pTmax),
                  enable_norm);
 
  
  /*   Create histograms to calculate the pT distributions   */
  unsigned int NbinsY  = 60;
  double Ymin = -5.0, Ymax = 5.0;
  
  UserPlain::phys(2, "b-bar Y distribution for b-quark events",
                  spacing<hist1d>::linear(NbinsY, Ymin, Ymax),
                  enable_norm);

  
  /*   We want the normalized distribution plot   */
  UserPlain::enable_plot(1);
  UserPlain::enable_plot(2);

  
  /*   The first plot is the normalized pT distribution */
  ltxfigure pTplot;
  
  pTplot.axis = "title = {$\\bar b$ $p_T$ distribution},"
  "xlabel={$p_T$ [GeV]}, ylabel={$1/\\sigma\\; d\\sigma/dp_T$ [$\\mathrm{GeV}^{-1}$]}, "
  "scaled ticks=false, yticklabel style={/pgf/number format/.cd, fixed,precision=5}, "
  "xmin=0, xmax=30, ymin=0, ymax=0.08";
  
  UserPlain::add_figure(1, pTplot);

  pTplot.axis = "title = {$\\bar b$ $y$ distribution},"
  "xlabel={$y$, ylabel={$1/\\sigma\\; d\\sigma/dy$},"
  "scaled ticks=false, yticklabel style={/pgf/number format/.cd, fixed,precision=5}, "
  "xmin=-5, xmax=5, ymin=0, ymax=0.3";
  
  UserPlain::add_figure(2, pTplot);
}




void UserBpT::userfunc(const shower_history<hhc>& s)
{
  /*   the shower stage after the shower evolution   */
  auto& sb = s.back();
  auto& p = sb.state;
  
  /*   momentum of the ee pair   */
  auto pee = p[-2].momentum + p[-3].momentum;
  
  if(pee.mag() > 400.0_GeV && abs(pee.rapidity()) < 2.0)
  {
    /*   Calculate the weight of the event   */
    double w = sb.weight*color_weight(sb);
    
    /*   Total cross section for pT and y distributions  */
    UserVoid::physfill(0, w);
    
    for(unsigned int i = 1; i <= p.number_of<qcd_out>(); i++)
      if(p[i].flavor == -5)
      {
        const auto& pbbar = p[i].momentum;
        double dy = pbbar.rapidity() - pee.rapidity();
        
        if(pbbar.perp() < 30_GeV) {
          UserPlain::normfill(1, w);
          UserPlain::physfill(1, w, dirac(), pbbar.perp());
          UserVoid::physfill(1, w);
        }
        
        if(std::abs(dy) < 5) {
          UserPlain::normfill(2, w);
          UserPlain::physfill(2, w, dirac(), dy);
          UserVoid::physfill(2, w);
        }
      } // End if (p[i].flavor == -5)
    
  } // End if (pee.mag() > 400.0 && abs(pee.rapidity())< 2)
}

