/*  File: analyser-ZpT.cc
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *  Modified by D. Soper on 17 December 2013.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
/*   std includes   */

/*   project headers  */

/*   local headers  */
#include "analyser-ZpT.h"

/*   other includes   */



void UserZpT::initfunc()
{
  /*   Creating object to calculate the total cross section   */
  UserVoid::phys(1, "Total cross section");
  
  /*   Creating a histogram to calculate the pT distribution   */
  unsigned int nbins = 100;
  double pTmin = 0.0_GeV, pTmax = 100.0_GeV;
  bool enable_norm = true;
  
  UserPlain::phys(1, "$Z/\\gamma$ $p_T$ distribution", spacing<hist1d>::linear(nbins, pTmin, pTmax), enable_norm);
  
  /*   We want to generate LaTeX plot for the pT distribution    */
  UserPlain::enable_plot(1);
  
  /*   The first plot is the normalized distribution */
  ltxfigure pTplot;
  
  pTplot.axis = "title = {$Z/\\gamma$ $p_T$ distribution},"
  "xlabel={$p_T$ [GeV]}, ylabel={$1/\\sigma\\; d\\sigma/dp_T$ [$\\mathrm{GeV}^{-1}$]}, "
  "scaled ticks=false, yticklabel style={/pgf/number format/.cd, fixed,precision=5}, "
  "xmin=0, xmax=100, ymin=0, ymax=0.04";

  UserPlain::add_figure(1, pTplot);
 
  /*   The next available default plot is the unnormalized distribution  */
  pTplot.axis = "title = {$Z/\\gamma$ $p_T$ distribution}, "
  "xlabel={$p_T$ [GeV]}, ylabel={$d\\sigma/dp_T$ [nb/GeV]}, "
  "xmin=0, xmax=100, ymin=0";
  
  UserPlain::add_figure(1, pTplot);
}


void UserZpT::userfunc(const shower_history<hhc>& s)
{
  /*   the shower stage after the shower evolution   */
  auto& sb = s.back();
  auto& p = sb.state;
  
  /*   Calculate the weight of the event   */
  constexpr double tonb = 389379.338;
  double w = sb.weight*color_weight(sb)*tonb;
  
  /*   Total cross section  */
  UserVoid::physfill(1, w);
  
  /*   momentum of the ee pair   */
  auto pee = p[-2].momentum + p[-3].momentum;
  
  /*   pT distribution   */
  if(pee.mag() > 400.0 && abs(pee.rapidity()) < 2.0 && pee.perp() < 100.0) {
    UserPlain::normfill(1, w);
    UserPlain::physfill(1, w, dirac(), pee.perp());
  }
}

