/*  File: analyser-Ninjet.cc
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *  Modified by D. Soper on 17 December 2013
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   std includes   */

/*   project headers  */

/*   local headers  */
#include "analyser-Ninjet.h"

/*   other includes   */

void UserNinjet::initfunc()
{
  /*   Create a histogram to calculate the one jet cross section   */
  unsigned int nbins = 20;
  double Nmin = 0.0001, Nmax = 20.0001;
  
  UserVoid::phys(1, "Number of jets");
  UserPlain::phys(1, "Number of partons in a jet", spacing<hist1d>::linear(nbins, Nmin, Nmax));
  
  cout<< "UserNinjet is using jet algorithm "
  <<jetDef.jet_algorithm()
  <<" with R = "<<jetDef.R()<<endl;
}



void UserNinjet::userfunc(const shower_history<hhc>& s)
{
  /*   the shower stage after the shower evolution   */
  auto& sb = s.back();
  auto& p = sb.state;
  
  /*   Calculate the weight of the event   */
  double w = sb.weight*color_weight(sb);
  
  /*   Converting deductor event to FastJet format   */
  make_pseudojets(theparticles, p);
  
  /*    Run fastjet algorithm   */
  ClusterSequence clustSeq1(theparticles, jetDef);
  
  /*    Extract inclusive jets (above pTjetmin)   */
  double pTjetmin = 200.0_GeV;
  incJets = clustSeq1.inclusive_jets(pTjetmin);
  
  /*    Fill histogram.   */
  unsigned int Njets = 0;
  
  /*    Only count jets that have |y| < 2.0   */
  for(unsigned int j = 0; j < incJets.size(); j++)
    if(abs(incJets[j].rap()) <= 2.0) {
      Njets++;
      constituents = incJets[j].constituents();
      UserPlain::physfill(1, w, dirac(), constituents.size());
    }
  
  UserVoid::physfill(1, w*Njets);
}

