/*  File: analyser-angle.cc
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *  Modified by D. Soper on 17 December 2013
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   std includes   */

/*   project headers  */

/*   local headers  */
#include "analyser-angle.h"

/*   other includes   */

void UserAngle::initfunc()
{
  /*   Total cross section in histogram  */
  UserVoid::phys(1,"Total cross section in histogram");
  
  /*   Create a histogram to calculate the one jet cross section
   *  phipi = phi/pi where phi is the azimuthal angle difference.
   */
  unsigned int nbins = 40;
  constexpr double phipimin = 2.0/3.0, phipimax = 1.0;
 
  UserPlain::phys(1, "Relative azimuthal angle distribution of the two leading jets",
                  spacing<hist1d>::linear(nbins, phipimin, phipimax));
  
  cout<<"UserAngle is using jet algorithm "
  <<jetDef.jet_algorithm()
  <<" with R = "<<jetDef.R() << endl;
}



void UserAngle::userfunc(const shower_history<hhc>& s)
{
  /*   the shower stage after the shower evolution   */
  auto& sb = s.back();
  auto& p = sb.state;
  
  /*   Converting deductor event to FastJet format   */
  make_pseudojets(theparticles,p);
  
  /*   Run fastjet algorithm   */
  ClusterSequence clustSeq(theparticles, jetDef);
  
  /*   Extract inclusive jets above pTjetmin   */
  double pTjetmin = 200.0_GeV;
  incJets = clustSeq.inclusive_jets(pTjetmin);
  
  /*   Sort them by pT.    */
  std::sort(incJets.begin(), incJets.end(),
            [](const PseudoJet& a, const PseudoJet& b)->bool {
              return a.perp() > b.perp();
            });
  
  /*   Select the jets with |y| < 2.  */
  OKjets.resize(0u);
  
  for(unsigned n = 0; n < incJets.size(); n++)
    if(std::abs(incJets[n].rap()) < 2.0) OKjets.push_back(incJets[n]);
  
  /*   We need at least two jets above 200 GeV.   */
  if(OKjets.size() < 2) return;
  
  /*   Calculate the weight of the event   */
  double w = sb.weight*color_weight(sb);
  
  /*    Find phi   */
  double phi = std::abs(OKjets[0].phi()-OKjets[1].phi())/M_PI;
  
  /*    Fill histogram   */
  UserVoid::physfill(1, w);
  UserPlain::physfill(1, w, dirac(), phi > 1.0 ? 2.0-phi : phi);
}



