/*  File: analysert.cc
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   std includes   */

/*   project headers  */

/*   local headers  */
#include "analyser-cms.h"

/*   other includes   */




void UserCMS::initfunc()
{
  auto bins = spacing<hist1d>::manual({0.0,1.0,1.5,2.0,2.5,3.0,3.5,4.0,4.5,5.0,5.5,6.0,7.0,8.0,9.0});

  /*   Create a histogram to calculate the one jet cross section   */
  phys(1, "Average number of jets", bins);
  phys(2, "Muller-Nevelet dijets ratio", bins);
  phys(3, "Inclusive, exclusive dijet ratio", bins);


  /*   We want to generate some LaTeX plots   */
  enable_plot(1);
  enable_plot(2);
  enable_plot(3);

  /*   Average number of jets   */
  ltxfigure pTplot;
  
  pTplot.axis = "xlabel={$\\Delta y$}, ylabel={$\\langle N\\rangle(\\Delta y)$},"
  "xmin=0, xmax=9, ymin=2, ymax=3";
  
  add_figure(1, pTplot);

  /*   Muller-Nevelet ratio   */
  pTplot.axis = "xlabel={$\\Delta y$}, ylabel={$R_{MN}(\\Delta y)$},"
  "xmin=0, xmax=9, ymin=1, ymax=1.8";
  
  add_figure(2, pTplot);

  /*   Inclusive-exclusive dijet ration  */
  pTplot.axis = "xlabel={$\\Delta y$}, ylabel={$R_{\\rm incl}(\\Delta y)$},"
  "xmin=0, xmax=9, ymin=1, ymax=1.8";
  
  add_figure(3, pTplot);
}



void UserCMS::userfunc(const shower_history<hhc>& s)
{
  /*   the shower stage after the shower evolution   */
  auto& sb = s.back();
  auto& p = sb.state;
  
  /*   Calculate the weight of the event   */
  constexpr double tonb = 389379.338;
  double wb = sb.weight*color_weight(sb)*tonb;
  
  /* preparing the momenta for FastJet */
  make_pseudojets(theparticles, p);
  
  /*   Clustering the partons using FastJet */
  ClusterSequence clustSeq(theparticles, jetDef);
  
  /*   Extract inclusive jets (pT and rapidity cut) */
  incJets = clustSeq.inclusive_jets(35.0_GeV);
  
  for(auto i = incJets.begin(); i < incJets.end();) {
    if(std::abs(i->rap()) > 4.7) i = incJets.erase(i);
    else i++;
  }

  /*   Needs at least 2 jets  */
  unsigned int nj = incJets.size();
  
  if(nj < 2) return;
  
  /*  sorting them by rapidity  */
  std::sort(incJets.begin(), incJets.end(),
            [](const PseudoJet& a, const PseudoJet& b)->bool {
              return a.rap() < b.rap();
            });
  
  /*   average pT of the most forward and backward has to be at least 60GeV  */
  PseudoJet &jetF = incJets.back(), &jetB = incJets.front();
    
  /*   Fill the histogram  */
  physfill(1, {{wb*nj, wb}}, dirac(), std::abs(jetF.rap()-jetB.rap()));
  physfill(2, {{wb, (nj == 2u ? wb : 0.0)}}, dirac(), std::abs(jetF.rap()-jetB.rap()));
  
  for(unsigned k = 0; k < nj; k++)
    for(unsigned l = k+1; l < nj; l++)
      physfill(3, {{wb, (nj == 2u ? wb : 0.0)}}, dirac(), std::abs(incJets[k].rap()-incJets[l].rap()));
  
}

