/*  File: analysert.cc
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   std includes   */

/*   project headers  */

/*   local headers  */
#include "analyser-jeppe.h"

/*   other includes   */




void UserJeppe::initfunc()
{
  /*   Create a histogram to calculate the one jet cross section   */
  phys(1, "Average number of jets", spacing<hist1d>::linear(28, 0.0, 8.0));
  
  /*   We want to generate some LaTeX plots   */
  enable_plot(1);
  
  /*   The first plot is the ratio   */
  ltxfigure pTplot;
  
  pTplot.axis = "xlabel={$\\Delta y$}, ylabel={$\\langle N\\rangle(\\Delta y)$},"
  "xmin=0, xmax=8, ymin=2, ymax=3";
  
  add_figure(1, pTplot);
  
  
  /*    The numerator and the denominator in one figure  */
  pTplot.axis = "xlabel={$\\Delta y$}, ylabel={$\\displaystyle\\int n_{\\rm jets}^\\beta\\, d\\sigma(\\Delta y)$},"
  "xmin=0, xmax=8, ymin=0";
  
  pTplot.add_plot2D("xm", "val2", "blue");
  pTplot.add_plot2D("xm", "val3", "green!60!black");
  
  add_figure(1, pTplot);
}



void UserJeppe::userfunc(const shower_history<hhc>& s)
{
  /*   the shower stage after the shower evolution   */
  auto& sb = s.back();
  auto& p = sb.state;
  
  /*   Calculate the weight of the event   */
  constexpr double tonb = 389379.338;
  double wb = sb.weight*color_weight(sb)*tonb;
  
  /* preparing the momenta for FastJet */
  make_pseudojets(theparticles, p);
  
  /*   Clustering the partons using FastJet */
  ClusterSequence clustSeq(theparticles, jetDef);
  
  /*   Extract inclusive jets (pT and rapidity cut) */
  incJets = clustSeq.inclusive_jets(30.0_GeV);
  
  for(auto i = incJets.begin(); i < incJets.end();) {
    if(std::abs(i->rap()) > 4.5) i = incJets.erase(i);
    else ++i;
  }

  /*   Needs at least 2 jets  */
  unsigned int nj = incJets.size();
  
  if(nj < 2) return;
  
  /*  sorting them by rapidity  */
  std::sort(incJets.begin(), incJets.end(),
            [](const PseudoJet& a, const PseudoJet& b)->bool {
              return a.rap() < b.rap();
            });
  
  /*   average pT of the most forward and backward has to be at least 60GeV  */
  PseudoJet &jetF = incJets.back(), &jetB = incJets.front();
  double pTf = jetF.perp(), pTb = jetB.perp();
  
  if(pTf+pTb < 120.0_GeV) return;
  
  /*   Jeppe's plot   */
  physfill(1, {{wb*nj, wb}}, dirac(), std::abs(jetF.rap()-jetB.rap()));
}

